/* Command-Line Interface
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
//#include <openspc.h>
#include "spc.h"

int main(int argc, char **argv)
{
	struct spc_format *spc = NULL;

	// check command-line args
	if(argc < 2) {
		printf("Please specify a .SPC file\n");
		return 1;
	}

	// read the file
	spc = read_spc_from_file(argv[1]);
	if(spc == NULL) {
		printf("Unable to read SPC.\n");
		return 1;
	}

	// print basic metadata
	if(spc->has_id666) {
		printf("Game: %s\nSong: %s\n\n", spc->id666_game_title, spc->id666_song_title);
	} else {
		printf("File: %s\n\n", argv[1]);
	}

	// parse interesting data from the sample table
	int num_samples = 0;
	struct sample_chart **chart = NULL;
	generate_sample_chart(spc, &num_samples, &chart);
	if(num_samples < 1 || chart == NULL) {
		printf("Unable to find any samples\n");
		return 0;
	}

	qsort(chart, num_samples, sizeof(struct sample_chart), sample_chart_comparator);

	// display chart of samples
	printf("Sample#\tStart\tLoop\tS Len\tL Len\n");
	printf("-------\t-------\t-------\t-------\t-------\n");
	struct sample_chart *c = (struct sample_chart*) chart;
	for(int i = 0; i < num_samples; i++) {
		printf("0x%X\t0x%X\t0x%X\t0x%X\t0x%X\n", c->sample_number, c->sample_address,
			c->loop_address, c->sample_length, c->loop_length);
		c++;
	}
	printf("\n");

	// cleanup
	free(chart);
	free(spc);
	return 0;
}
