#ifndef __SPC_H__
#define __SPC_H__

/* SPC-related functions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

#ifndef u8
typedef unsigned char u8;
#endif
#ifndef u16
typedef unsigned short u16;
#endif
#ifndef u32
typedef unsigned int u32;
#endif

#pragma pack(1)
struct spc_format {
	char header_constant[33];
	u8 twentysix_twentysix[2];
	u8 has_id666;
	u8 version_minor;

	u16 reg_pc;
	u8 reg_a;
	u8 reg_x;
	u8 reg_y;
	u8 reg_psw;
	u8 reg_sp;
	u16 reg_reserved;

	char id666_song_title[32];
	char id666_game_title[32];
	char id666_dumper_name[16];
	char id666_comments[32];
	char id666_date_dumped[11]; /* mm/dd/yyyy */
	char id666_seconds_before_fadeout[3];
	char id666_fade_length_in_msec[5];
	char id666_artist_name[32];
	u8 id666_default_channel_disabled;
	u8 id666_emulator_used_to_dump; /* 1=zsnes, 2=snes9x */
	char id666_reserved[45];

	u8 aram[65536];
	u8 dsp_registers[128];
	u8 unused[64];
	u8 extra_ram[64];
};

struct sample_chart {
	int sample_number;
	u16 sample_address;
	u16 loop_address;
	int sample_length;
	int loop_length;
};

struct spc_format *read_spc_from_file(char *filename);
void generate_sample_chart(struct spc_format *spc, int *out_num_samples, struct sample_chart ***out_chart);
int sample_chart_comparator(const void *raw_a, const void *raw_b);
void dump_sample(struct spc_format *spc, u16 address, int length, char *filename);
void play_spc(struct spc_format *spc);
void stop_spc(void);

#endif
